from fastapi import FastAPI, Header
from fastapi.responses import ORJSONResponse
import json
from typing import Union, List
import requests
from sredis.sredis import * 
import logging
from models import User, DataCenter, Token
from uuid import uuid4

logging.config.fileConfig('logging.conf', disable_existing_loggers=False)

# get root logger
logger = logging.getLogger(__name__)

app = FastAPI()
add_datacenter("https://kappa1.fured.cloud.bme.hu")
add_datacenter("https://kappa2.fured.cloud.bme.hu")
add_datacenter("https://kappa3.fured.cloud.bme.hu")

def _proxy_datacenters(serverpath: str, username, method="GET", balancer_fun = rr_get):
    server = balancer_fun()
    token = get_datacenter_token(username, server)
    url=f"{server}/{serverpath}"
    logger.debug("Req: " + url)
    t_resp = requests.request(
        method=method,
        url=url,
        allow_redirects=False, verify=False, 
        headers={
            'Authorization': token
        }
    )
    
    response = ORJSONResponse(
        json.loads(t_resp.content), 
        status_code=t_resp.status_code
    )
    return response

@app.get("/lb/{server_path:path}")
def proxy(
        Authorization: Union[str, None] = Header(default=None), 
        server_path: str = "/"
    ):
    username = get_user(str(Authorization))
    if not username:
        return Response(status_code=401)
    
    return _proxy_datacenters(server_path, username)

@app.post("/lb/{server_path:path}")
def proxy(
        Authorization: Union[str, None] = Header(default=None),
        server_path: str = "/"
    ):
    username = get_user(str(Authorization))
    if not username:
        return Response(status_code=401)
    return _proxy_datacenters(server_path, username, method="POST")

@app.post("/add_datacenter/")
def create_datacenter(
        Authorization: Union[str, None] = Header(default=None),
        dc: DataCenter = None
    ):
    username = get_user(str(Authorization))
    if not username:
        return Response(status_code=401)
    add_datacenter(dc.name)
    return Response(status_code=201)


@app.post("/create_user/")
def create_user(
        user: User = None
    ):
    user.token = str(uuid4())
    new_user(user.username, user.token)
    return user

@app.post("/set_tokens/")
def set_tokens(
        Authorization: Union[str, None] = Header(default=None),
        tokens: List[Token] = None
    ):
    username = get_user(str(Authorization))
    if not username:
        return Response(status_code=401)
    for token in tokens:
        set_token(username, str(token.datacenter), str(token.token))
    return tokens
